@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -191,34 +191,35 @@ int CItemData::Write(PWSfile *out) const
     WriteIfSet(TextFields[i], out, true);
 
   int32 i32;
+  
   for (i = 0; TimeFields[i] != END; i++) {
     time_t t = 0;
     GetTime(TimeFields[i], t);
     if (t != 0) {
-      i32 = static_cast<int>(t);
+      i32 = static_cast<int32>(t);
       out->WriteField(static_cast<unsigned char>(TimeFields[i]),
-                      reinterpret_cast<unsigned char *>(&i32), sizeof(int32));
+                      reinterpret_cast<unsigned char *>(&i32), sizeof(i32));
     }
   }
 
   GetXTimeInt(i32);
   if (i32 > 0 && i32 <= 3650) {
-    out->WriteField(XTIME_INT, reinterpret_cast<unsigned char *>(&i32), sizeof(int32));
+    out->WriteField(XTIME_INT, reinterpret_cast<unsigned char *>(&i32), sizeof(i32));
   }
 
   GetKBShortcut(i32);
   if (i32 != 0) {
     out->WriteField(KBSHORTCUT, reinterpret_cast<unsigned char *>(&i32),
-                    sizeof(int32));
+                    sizeof(i32));
   }
 
-  short i16;
+  int16 i16;
   GetDCA(i16);
   if (i16 >= PWSprefs::minDCA && i16 <= PWSprefs::maxDCA)
-    out->WriteField(DCA, reinterpret_cast<unsigned char *>(&i16), sizeof(short));
+    out->WriteField(DCA, reinterpret_cast<unsigned char *>(&i16), sizeof(i16));
   GetShiftDCA(i16);
   if (i16 >= PWSprefs::minDCA && i16 <= PWSprefs::maxDCA)
-    out->WriteField(SHIFTDCA, reinterpret_cast<unsigned char *>(&i16), sizeof(short));
+    out->WriteField(SHIFTDCA, reinterpret_cast<unsigned char *>(&i16), sizeof(i16));
 
   WriteIfSet(PROTECTED, out, false);
 
@@ -300,7 +301,7 @@ StringX CItemData::GetFieldValue(FieldType ft) const
       return GetATimeL();
     case XTIME:      /* 0a */
       {
-        int xint(0);
+        int32 xint(0);
         str = GetXTimeL();
         GetXTimeInt(xint);
         if (xint != 0)
@@ -391,7 +392,7 @@ void CItemData::GetTime(int whichtime, time_t &t) const
     GetField(fiter->second, in, tlen);
 
     if (tlen != 0) {
-      int t32;
+      int32 t32;
       ASSERT(tlen == sizeof(t32));
       memcpy(&t32, in, sizeof(t32));
       t = t32;
@@ -496,7 +497,7 @@ bool CItemData::IsProtected() const
   return ucprotected != 0;
 }
 
-void CItemData::GetDCA(short &iDCA, const bool bShift) const
+void CItemData::GetDCA(int16 &iDCA, const bool bShift) const
 {
   FieldConstIter fiter = m_fields.find(bShift ? SHIFTDCA : DCA);
   if (fiter != m_fields.end()) {
@@ -505,8 +506,8 @@ void CItemData::GetDCA(short &iDCA, const bool bShift) const
     GetField(fiter->second, in, tlen);
 
     if (tlen != 0) {
-      ASSERT(tlen == sizeof(short));
-      memcpy(&iDCA, in, sizeof(short));
+      ASSERT(tlen == sizeof(int16));
+      memcpy(&iDCA, in, sizeof(int16));
     } else {
       iDCA = -1;
     }
@@ -516,7 +517,7 @@ void CItemData::GetDCA(short &iDCA, const bool bShift) const
 
 StringX CItemData::GetDCA(const bool bShift) const
 {
-  short dca;
+  int16 dca;
   GetDCA(dca, bShift);
   oStringXStream os;
   os << dca;
@@ -857,8 +858,8 @@ string CItemData::GetXML(unsigned id, const FieldBits &bsExport,
   oss << "\t\t<uuid><![CDATA[" << GetUUID() << "]]></uuid>" << endl;
 
   time_t t;
-  int i32;
-  short i16;
+  int32 i32;
+  int16 i16;
 
   GetCTime(t);
   if (bsExport.test(CItemData::CTIME) && t)
@@ -1150,7 +1151,7 @@ void CItemData::UpdatePassword(const StringX &password)
   time(&t);
   SetPMTime(t);
 
-  int xint;
+  int32 xint;
   GetXTimeInt(xint);
   if (xint != 0) {
     // convert days to seconds for time_t
@@ -1218,11 +1219,11 @@ void CItemData::UpdatePasswordHistory()
   // Now create string version!
   StringX new_PWHistory, buffer;
 
-  Format(new_PWHistory, _T("1%02x%02x"), pwh_max, num);
+  Format(new_PWHistory, L"1%02x%02x", pwh_max, num);
 
   PWHistList::iterator iter;
   for (iter = pwhistlist.begin(); iter != pwhistlist.end(); iter++) {
-    Format(buffer, _T("%08x%04x%s"),
+    Format(buffer, L"%08x%04x%ls",
            static_cast<long>(iter->changetttdate), iter->password.length(),
            iter->password.c_str());
     new_PWHistory += buffer;
@@ -1297,7 +1298,7 @@ void CItemData::SetTime(int whichtime)
 
 void CItemData::SetTime(int whichtime, time_t t)
 {
-  int t32 = static_cast<int>(t);
+  int t32 = static_cast<int32>(t);
   SetField(static_cast<FieldType>(whichtime),
            reinterpret_cast<const unsigned char *>(&t32), sizeof(t32));
 }
@@ -1334,7 +1335,7 @@ void CItemData::SetXTimeInt(int32 &xint)
 
 bool CItemData::SetXTimeInt(const stringT &xint_str)
 {
-  int xint(0);
+  int32 xint(0);
 
   if (xint_str.empty()) {
     SetXTimeInt(xint);
@@ -1426,15 +1427,15 @@ void CItemData::SetPolicyName(const StringX &sx_PolicyName)
   SetField(POLICYNAME, sx_PolicyName);
 }
 
-void CItemData::SetDCA(const short &iDCA, const bool bShift)
+void CItemData::SetDCA(const int16 &iDCA, const bool bShift)
 {
    SetField(bShift ? SHIFTDCA : DCA,
-            reinterpret_cast<const unsigned char *>(&iDCA), sizeof(short));
+            reinterpret_cast<const unsigned char *>(&iDCA), sizeof(int16));
 }
 
 bool CItemData::SetDCA(const stringT &cs_DCA, const bool bShift)
 {
-  short iDCA(-1);
+  int16 iDCA(-1);
 
   if (cs_DCA.empty()) {
     SetDCA(iDCA, bShift);
@@ -1606,7 +1607,7 @@ bool CItemData::ValidatePWHistory()
 
   PWHistList::const_iterator citer;
   for (citer = pwhistlist.begin(); citer != pwhistlist.end(); citer++) {
-    Format(sxBuffer, _T("%08x%04x%s"),
+    Format(sxBuffer, L"%08x%04x%ls",
              static_cast<long>(citer->changetttdate), citer->password.length(),
              citer->password.c_str());
       sxNewHistory += sxBuffer;
@@ -1689,13 +1690,10 @@ bool CItemData::Matches(int num1, int num2, int iObject,
     return PWSMatch::Match(num1, num2, iValue, iFunction);
 }
 
-bool CItemData::Matches(short dca, int iFunction, const bool bShift) const
+bool CItemData::Matches(int16 dca, int iFunction, const bool bShift) const
 {
-  short iDCA;
+  int16 iDCA;
   GetDCA(iDCA, bShift);
-  if (iDCA < 0)
-    iDCA = static_cast<short>(PWSprefs::GetInstance()->GetPref(bShift ?
-               PWSprefs::ShiftDoubleClickAction : PWSprefs::DoubleClickAction));
 
   switch (iFunction) {
     case PWSMatch::MR_IS:
@@ -1884,10 +1882,10 @@ static bool pull_int32(int32 &i, const unsigned char *data, size_t len)
   return true;
 }
 
-static bool pull_int16(short &i16, const unsigned char *data, size_t len)
+static bool pull_int16(int16 &i16, const unsigned char *data, size_t len)
 {
-  if (len == sizeof(short)) {
-    i16 = *reinterpret_cast<const short *>(data);
+  if (len == sizeof(int16)) {
+    i16 = *reinterpret_cast<const int16 *>(data);
   } else {
     ASSERT(0);
     return false;
@@ -1940,8 +1938,8 @@ bool CItemData::SetField(int type, const unsigned char *data, size_t len)
 {
   StringX str;
   time_t t;
-  int i32;
-  short i16;
+  int32 i32;
+  int16 i16;
   unsigned char uc;
 
   FieldType ft = static_cast<FieldType>(type);
@@ -2033,7 +2031,7 @@ static void push_string(vector<char> &v, char type,
       v.insert(v.end(), reinterpret_cast<const char *>(utf8),
                reinterpret_cast<const char *>(utf8) + utf8Len);
     } else
-      pws_os::Trace(_T("ItemData.cpp: push_string(%s): ToUTF8 failed!\n"), str.c_str());
+      pws_os::Trace(_T("ItemData.cpp: push_string(%ls): ToUTF8 failed!\n"), str.c_str());
   }
 }
 
@@ -2058,13 +2056,13 @@ static void push_int32(vector<char> &v, char type, int32 i)
   }
 }
 
-static void push_int16(vector<char> &v, char type, short i)
+static void push_int16(vector<char> &v, char type, int16 i)
 {
   if (i != 0) {
     v.push_back(type);
-    push_length(v, sizeof(short));
+    push_length(v, sizeof(int16));
     v.insert(v.end(),
-      reinterpret_cast<char *>(&i), reinterpret_cast<char *>(&i) + sizeof(short));
+      reinterpret_cast<char *>(&i), reinterpret_cast<char *>(&i) + sizeof(int16));
   }
 }
 
@@ -2084,7 +2082,7 @@ void CItemData::SerializePlainText(vector<char> &v,
   uuid_array_t uuid_array;
   time_t t = 0;
   int32 i32 = 0;
-  short i16 = 0;
+  int16 i16 = 0;
   unsigned char uc = 0;
 
   v.clear();
