@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2012 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -222,6 +222,8 @@ static string GetFilterXML(const st_filters &filters, bool bWithFormatting)
 
     const int ft = static_cast<int>(st_fldata.ftype);
     const char *pszfieldtype = {"\0"};
+
+    // These are the entry names exported and must be recognised by the associated schema
     switch (ft) {
       case FT_GROUPTITLE:
         pszfieldtype = "grouptitle";
@@ -266,7 +268,10 @@ static string GetFilterXML(const st_filters &filters, bool bWithFormatting)
         pszfieldtype = "symbols";
         break;
       case FT_POLICYNAME:
-        pszfieldtype = "policyname";
+        pszfieldtype = "policy_name";
+        break;
+      case FT_KBSHORTCUT:
+        pszfieldtype = "kbshortcut";
         break;
       // Time fields
       case FT_CTIME:
@@ -528,7 +533,6 @@ std::string PWSFilters::GetFilterXMLHeader(const StringX &currentfile,
   size_t utf8Len = 0;
 
   ostringstream oss;
-  StringX tmp;
   stringT cs_tmp;
   time_t time_now;
 
@@ -562,7 +566,7 @@ std::string PWSFilters::GetFilterXMLHeader(const StringX &currentfile,
     if (!hdr.m_lastsavedby.empty() || !hdr.m_lastsavedon.empty()) {
       stringT wls(_T(""));
       Format(wls,
-             _T("%s on %s"),
+             L"%ls on %ls",
              hdr.m_lastsavedby.c_str(), hdr.m_lastsavedon.c_str());
       utf8conv.ToUTF8(wls.c_str(), utf8, utf8Len);
       oss << "WhoSaved=\"";
@@ -678,22 +682,22 @@ stringT PWSFilters::GetFilterDescription(const st_FilterRow &st_fldata)
   switch (st_fldata.mtype) {
     case PWSMatch::MT_PASSWORD:
       if (st_fldata.rule == PWSMatch::MR_EXPIRED) {
-        Format(cs_criteria, _T("%s"), cs_rule.c_str());
+        Format(cs_criteria, L"%ls", cs_rule.c_str());
         break;
       } else if (st_fldata.rule == PWSMatch::MR_WILLEXPIRE) {
-        Format(cs_criteria, _T("%s %s"), cs_rule.c_str(), cs1.c_str());
+        Format(cs_criteria, L"%ls %ls", cs_rule.c_str(), cs1.c_str());
         break;
       }
       // Note: purpose drop through to standard 'string' processing
     case PWSMatch::MT_STRING:
       if (st_fldata.rule == PWSMatch::MR_PRESENT ||
           st_fldata.rule == PWSMatch::MR_NOTPRESENT)
-        Format(cs_criteria, _T("%s"), cs_rule.c_str());
+        Format(cs_criteria, L"%ls", cs_rule.c_str());
       else {
-        stringT cs_delim(_T(""));
-        if (cs1.find(_T(" ")) != stringT::npos)
-          cs_delim = _T("'");
-        Format(cs_criteria, _T("%s %s%s%s %s"), 
+        stringT cs_delim(L"");
+        if (cs1.find(L" ") != stringT::npos)
+          cs_delim = L"'";
+        Format(cs_criteria, L"%ls %ls%ls%ls %ls", 
                cs_rule.c_str(), cs_delim.c_str(), 
                cs1.c_str(), cs_delim.c_str(), cs2.c_str());
       }
@@ -703,14 +707,14 @@ stringT PWSFilters::GetFilterDescription(const st_FilterRow &st_fldata)
     case PWSMatch::MT_DATE:
       if (st_fldata.rule == PWSMatch::MR_PRESENT ||
           st_fldata.rule == PWSMatch::MR_NOTPRESENT)
-        Format(cs_criteria, _T("%s"), cs_rule.c_str());
+        Format(cs_criteria, L"%ls", cs_rule.c_str());
       else
       if (st_fldata.rule == PWSMatch::MR_BETWEEN) {  // Date or Integer only
         LoadAString(cs_and, IDSC_AND);
-        Format(cs_criteria, _T("%s %s %s %s"), 
+        Format(cs_criteria, L"%ls %ls %ls %ls", 
                cs_rule.c_str(), cs1.c_str(), cs_and.c_str(), cs2.c_str());
       } else {
-        Format(cs_criteria, _T("%s %s"), cs_rule.c_str(), cs1.c_str());
+        Format(cs_criteria, L"%ls %ls", cs_rule.c_str(), cs1.c_str());
       }
       if (st_fldata.mtype == PWSMatch::MT_DATE &&
           st_fldata.fdatetype == 1 /* Relative */) {
@@ -747,13 +751,13 @@ stringT PWSFilters::GetFilterDescription(const st_FilterRow &st_fldata)
       break;
     case PWSMatch::MT_DCA:
     case PWSMatch::MT_SHIFTDCA:
-      Format(cs_criteria, _T("%s %s"), cs_rule.c_str(), cs1.c_str());
+      Format(cs_criteria, L"%ls %ls", cs_rule.c_str(), cs1.c_str());
       break;
     case PWSMatch::MT_ENTRYTYPE:
-      Format(cs_criteria, _T("%s %s"), cs_rule.c_str(), cs1.c_str());
+      Format(cs_criteria, L"%ls %ls", cs_rule.c_str(), cs1.c_str());
       break;
     case PWSMatch::MT_ENTRYSTATUS:
-      Format(cs_criteria, _T("%s %s"), cs_rule.c_str(), cs1.c_str());
+      Format(cs_criteria, L"%ls %ls", cs_rule.c_str(), cs1.c_str());
       break;
     default:
       ASSERT(0);
