@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -136,12 +136,13 @@ public:
     USER_EXIT,                                //  15
     XML_FAILED_VALIDATION,                    //  16
     XML_FAILED_IMPORT,                        //  17
-    LIMIT_REACHED,                            //  18
+    LIMIT_REACHED,                            //  18 - OBSOLETE (for demo)
     UNIMPLEMENTED,                            //  19
     NO_ENTRIES_EXPORTED,                      //  20
     DB_HAS_DUPLICATES,                        //  21
     OK_WITH_ERRORS,                           //  22
-    OK_WITH_VALIDATION_ERRORS                 //  23
+    OK_WITH_VALIDATION_ERRORS,                //  23
+    OPEN_NODB                                 //  24
   };
 
   PWScore();
@@ -195,10 +196,13 @@ public:
   int WriteCurFile() {return WriteFile(m_currfile);}
   int WriteFile(const StringX &filename, const bool bUpdateSig = true,
                 PWSfile::VERSION version = PWSfile::VCURRENT);
+  int WriteExportFile(const StringX &filename, OrderedItemList *pOIL,
+                      PWScore *pINcore, CReport *pRpt = NULL,
+                      PWSfile::VERSION version = PWSfile::VCURRENT);
   int WriteV17File(const StringX &filename)
-  {return WriteFile(filename, true, PWSfile::V17);}
+  {return WriteFile(filename, false, PWSfile::V17);}
   int WriteV2File(const StringX &filename)
-  {return WriteFile(filename, true, PWSfile::V20);}
+  {return WriteFile(filename, false, PWSfile::V20);}
 
   // R/O file status
   void SetReadOnly(bool state) {m_IsReadOnly = state;}
@@ -214,7 +218,7 @@ public:
                     const stringT &subgroup_name,
                     const int &subgroup_object,
                     const int &subgroup_function,
-                    const OrderedItemList *il);
+                    const OrderedItemList *pOIL);
 
   void Compare(PWScore *pothercore,
                const CItemData::FieldBits &bsFields, const bool &subgroup_bset,
@@ -241,14 +245,14 @@ public:
                          const CItemData::FieldBits &bsExport,
                          const stringT &subgroup, const int &iObject,
                          const int &iFunction, const TCHAR &delimiter,
-                         int &numExported, const OrderedItemList *il = NULL,
+                         int &numExported, const OrderedItemList *pOIL = NULL,
                          CReport *pRpt = NULL);
 
   int WriteXMLFile(const StringX &filename,
                    const CItemData::FieldBits &bsExport,
                    const stringT &subgroup, const int &iObject,
                    const int &iFunction, const TCHAR &delimiter,
-                   int &numExported, const OrderedItemList *il = NULL,
+                   int &numExported, const OrderedItemList *pOIL = NULL,
                    const bool &bFilterActive = false,
                    CReport *pRpt = NULL);
 
@@ -426,6 +430,7 @@ public:
   bool WasDisplayStatusChanged() const;
 
   const PWSfile::HeaderRecord &GetHeader() const {return m_hdr;}
+  void SetHeader(const PWSfile::HeaderRecord &hdr) { m_hdr = hdr; }
   void GetDBProperties(st_DBProperties &st_dbp);
   void SetHeaderUserFields(st_DBProperties &st_dbp);
 
@@ -525,8 +530,7 @@ private:
   // End of Command Interface implementations
 
   // Validate() returns true if data modified, false if all OK
-  bool Validate(const size_t iMAXCHARS, const bool bInReadfile,
-                CReport *pRpt, st_ValidateResults &st_vr);
+  bool Validate(const size_t iMAXCHARS, CReport *pRpt, st_ValidateResults &st_vr);
 
   void ParseDependants(); // populate data structures as needed - called in ReadFile()
   void ResetAllAliasPasswords(const pws_os::CUUID &base_uuid);
@@ -640,7 +644,7 @@ private:
   void RemoveExpiryEntry(const CItemData &ci)
   {m_ExpireCandidates.Remove(ci);}
 
-  stringT GetXMLPWPolicies();
+  stringT GetXMLPWPolicies(const OrderedItemList *pOIL = NULL);
   PSWDPolicyMap m_MapPSWDPLC;
 
   KBShortcutMap m_KBShortcutMap;
