@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -46,19 +46,18 @@ CAddEdit_Additional::CAddEdit_Additional(CWnd * pParent, st_AE_master_data *pAEM
                            GetPref(PWSprefs::NumPWHistoryDefault);
 
   // Save PWS HotKey info
-  int32 iPWSHotKeyValue = int32(PWSprefs::GetInstance()->GetPref(PWSprefs::HotKey));
-  m_wVirtualKeyCode = iPWSHotKeyValue & 0xff;
-  WORD wHKModifiers = iPWSHotKeyValue >> 16;
-  WORD wPWSModifiers = ConvertModifersMFC2PWS(wHKModifiers);
-  m_wModifiers = ConvertModifersMFC2Windows(wHKModifiers);
+  int32 iAppHotKeyValue = int32(PWSprefs::GetInstance()->GetPref(PWSprefs::HotKey));
+  WORD wHKModifiers = iAppHotKeyValue >> 16;
+  m_wAppVirtualKeyCode = iAppHotKeyValue & 0xff;
+  m_wAppWindowsModifiers = ConvertModifersMFC2Windows(wHKModifiers);
 
   // Can't be enabled if not set!
-  if (iPWSHotKeyValue == 0) {
-    m_bPWSHotKeyEnabled = false;
-    m_iPWSAppHotKey = 0;
+  if (iAppHotKeyValue == 0) {
+    m_bAppHotKeyEnabled = false;
+    m_iAppHotKey = 0;
   } else {
-    m_bPWSHotKeyEnabled = PWSprefs::GetInstance()->GetPref(PWSprefs::HotKeyEnabled);
-    m_iPWSAppHotKey = (wPWSModifiers << 16) + m_wVirtualKeyCode;
+    m_bAppHotKeyEnabled = PWSprefs::GetInstance()->GetPref(PWSprefs::HotKeyEnabled);
+    m_iAppHotKey = (m_wAppWindowsModifiers << 16) + m_wAppVirtualKeyCode;
   }
 }
 
@@ -328,7 +327,7 @@ void CAddEdit_Additional::SetupDCAComboBoxes(CComboBox *pcbox, bool isShift)
       cs_text.LoadString(ResPref[i].res);
       if (ResPref[i].pref == DefaultDCA) {
         const CString cs_default(MAKEINTRESOURCE(IDSC_DEFAULT));
-        cs_text += L" ("; cs_text += cs_default; cs_text += L")";
+        cs_text += cs_default;
       }
       int nIndex = pcbox->AddString(cs_text);
       pcbox->SetItemData(nIndex, ResPref[i].pref);
@@ -440,25 +439,29 @@ BOOL CAddEdit_Additional::OnKillActive()
 
 void CAddEdit_Additional::OnEntryHotKeyKillFocus()
 {
-  if (m_bPWSHotKeyEnabled)
+  if (m_bAppHotKeyEnabled || m_wAppWindowsModifiers == 0 || m_wAppVirtualKeyCode == 0)
     return;
 
+  // If PWS Application had an active Hot Key before the user edited
+  // this entry's Hot Key, put it back
   BOOL brc = RegisterHotKey(GetMainDlg()->m_hWnd, PWS_HOTKEY_ID,
-                            UINT(m_wModifiers), UINT(m_wVirtualKeyCode));
-  ASSERT(brc);
-
-  m_bPWSHotKeyEnabled = true;
+                      UINT(m_wAppWindowsModifiers), UINT(m_wAppVirtualKeyCode));
+  if (brc)
+    m_bAppHotKeyEnabled = true;
 }
 
 void CAddEdit_Additional::OnEntryHotKeySetFocus()
 {
-  if (!m_bPWSHotKeyEnabled)
+  if (!m_bAppHotKeyEnabled)
     return;
   
+  // If PWS Application has an active Hot Key, disable it when user
+  // is potentially editing this entry's Hot Key
   BOOL brc = UnregisterHotKey(GetMainDlg()->m_hWnd, PWS_HOTKEY_ID);
+
   ASSERT(brc);
   
-  m_bPWSHotKeyEnabled = false;
+  m_bAppHotKeyEnabled = false;
 }
 
 int CAddEdit_Additional::CheckKeyboardShortcut()
@@ -498,7 +501,7 @@ int CAddEdit_Additional::CheckKeyboardShortcut()
       return KBSHORTCUT_INVALID_CHARACTER;
     }
 
-    if (m_iPWSAppHotKey == iKBShortcut) {
+    if (m_iAppHotKey == iKBShortcut) {
       // Same as PWS application HotKey
       cs_msg.LoadString(IDS_KBS_SAMEASAPP);
       m_stc_warning.SetWindowText(cs_msg);
@@ -570,7 +573,6 @@ int CAddEdit_Additional::CheckKeyboardShortcut()
       const StringX sxGroup = iter->second.GetGroup();
       const StringX sxTitle = iter->second.GetTitle();
       const StringX sxUser  = iter->second.GetUser();
-      CString cs_errmsg;
       cs_errmsg.Format(IDS_KBS_INUSEBYENTRY, cs_HotKey,
                        sxGroup.c_str(), sxTitle.c_str(), sxUser.c_str());
       m_stc_warning.SetWindowText(cs_errmsg);
@@ -595,7 +597,7 @@ int CAddEdit_Additional::CheckKeyboardShortcut()
       // Warn user that it is already in use for a menu item
       // (on this instance for this user!)
       Remove(sxMenuItemName, L'&');
-      CString cs_errmsg, cs_override(MAKEINTRESOURCE(IDS_ENTRYKBST_OVERRIDE));
+      CString cs_override(MAKEINTRESOURCE(IDS_ENTRYKBST_OVERRIDE));
       cs_errmsg.Format(IDS_KBS_INUSEBYMENU, cs_HotKey, sxMenuItemName.c_str(), cs_override);
       m_stc_warning.SetWindowText(cs_errmsg);
       m_stc_warning.ShowWindow(SW_SHOW);
@@ -656,7 +658,6 @@ BOOL CAddEdit_Additional::OnApply()
     return FALSE; //CAddEdit_PropertyPage::OnApply();
 
   CWnd *pFocus(NULL);
-  CGeneralMsgBox gmb;
 
   UpdateData(TRUE);
   M_autotype().EmptyIfOnlyWhiteSpace();
@@ -679,10 +680,11 @@ BOOL CAddEdit_Additional::OnApply()
     bool bAutoType(false);
     StringX sxAutotype(L"");
     bool bURLSpecial;
-    PWSAuxParse::GetExpandedString(M_runcommand(), L"", NULL,
+    PWSAuxParse::GetExpandedString(M_runcommand(), L"", NULL, NULL,
                                    bAutoType, sxAutotype, errmsg, st_column,
                                    bURLSpecial);
     if (errmsg.length() > 0) {
+      CGeneralMsgBox gmb;
       CString cs_title(MAKEINTRESOURCE(IDS_RUNCOMMAND_ERROR));
       CString cs_temp(MAKEINTRESOURCE(IDS_RUN_IGNOREORFIX));
       CString cs_errmsg;
@@ -720,6 +722,7 @@ BOOL CAddEdit_Additional::OnApply()
 
   if (M_SavePWHistory() == TRUE &&
       (M_MaxPWHistory() < 1 || M_MaxPWHistory() > 255)) {
+    CGeneralMsgBox gmb;
     gmb.AfxMessageBox(IDS_DEFAULTNUMPWH);
     pFocus = GetDlgItem(IDC_MAXPWHISTORY);
     goto error;
@@ -815,9 +818,15 @@ void CAddEdit_Additional::OnSTCExClicked(UINT nID)
         std::wstring errmsg;
         size_t st_column;
         bool bURLSpecial;
+
+        CItemData *pbci = NULL;
+        if (M_pci()->IsAlias()) {
+          pbci = M_pcore()->GetBaseEntry(M_pci());
+        }
+
         sxData = PWSAuxParse::GetExpandedString(M_runcommand(),
                                                  M_currentDB(),
-                                                 M_pci(),
+                                                 M_pci(), pbci,
                                                  GetMainDlg()->m_bDoAutoType,
                                                  GetMainDlg()->m_sxAutoType,
                                                  errmsg, st_column, bURLSpecial);
