@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -127,7 +127,6 @@ void CPasskeySetup::OnOK()
     CString cs_msg, cs_text;
     cs_msg.Format(IDS_WEAKPASSPHRASE, errmess.c_str());
 #ifndef PWS_FORCE_STRONG_PASSPHRASE
-    CGeneralMsgBox gmb;
     cs_text.LoadString(IDS_USEITANYWAY);
     cs_msg += cs_text;
     INT_PTR rc = gmb.AfxMessageBox(cs_msg, NULL, MB_YESNO | MB_ICONSTOP);
@@ -232,6 +231,9 @@ LRESULT CPasskeySetup::OnInsertBuffer(WPARAM, LPARAM)
   m_pSecCtl->SetSel(nStartChar + vkbuffer.GetLength(),
                     nStartChar + vkbuffer.GetLength());
 
+  // Make us on top
+  SetWindowPos(&wndTop, 0, 0, 0, 0, SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE);
+
   return 0L;
 }
 
