@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -39,6 +39,7 @@ static char THIS_FILE[] = __FILE__;
   
 */
 
+// nonHex* are irrelevant and disabled when hex is selected
 const UINT CPasswordPolicyDlg::nonHex[CPasswordPolicyDlg::N_NOHEX] = {
   IDC_USELOWERCASE, IDC_USEUPPERCASE, IDC_USEDIGITS,
   IDC_USESYMBOLS, IDC_EASYVISION, IDC_PRONOUNCEABLE};
@@ -132,7 +133,7 @@ void CPasswordPolicyDlg::DoDataExchange(CDataExchange* pDX)
 
     DDX_Control(pDX, IDC_OWNSYMBOLS, (CEdit&)m_SymbolsEdit);
 
-    // Because we can show the generated password when used from Mangage->Generate
+    // Because we can show the generated password when used from Manage->Generate
     DDX_Control(pDX, IDC_PASSWORD, m_ex_password);
 
     DDX_Control(pDX, IDC_POLICYNAME, m_PolicyNameEdit);
@@ -141,32 +142,32 @@ void CPasswordPolicyDlg::DoDataExchange(CDataExchange* pDX)
 }
 
 BEGIN_MESSAGE_MAP(CPasswordPolicyDlg, CPWDialog)
-//{{AFX_MSG_MAP(CPasswordPolicyDlg)
-ON_BN_CLICKED(IDOK, OnOK)
-ON_BN_CLICKED(IDCANCEL, OnCancel)
-ON_BN_CLICKED(ID_HELP, OnHelp)
+  //{{AFX_MSG_MAP(CPasswordPolicyDlg)
+  ON_BN_CLICKED(IDOK, OnOK)
+  ON_BN_CLICKED(IDCANCEL, OnCancel)
+  ON_BN_CLICKED(ID_HELP, OnHelp)
 
-ON_BN_CLICKED(IDC_USEHEXDIGITS, OnUseHexdigits)
-ON_BN_CLICKED(IDC_USELOWERCASE, OnUseLowerCase)
-ON_BN_CLICKED(IDC_USEUPPERCASE, OnUseUpperCase)
-ON_BN_CLICKED(IDC_USEDIGITS, OnUseDigits)
-ON_BN_CLICKED(IDC_USESYMBOLS, OnUseSymbols)
-ON_BN_CLICKED(IDC_EASYVISION, OnEasyVision)
-ON_BN_CLICKED(IDC_PRONOUNCEABLE, OnMakePronounceable)
+  ON_BN_CLICKED(IDC_USEHEXDIGITS, OnUseHexdigits)
+  ON_BN_CLICKED(IDC_USELOWERCASE, OnUseLowerCase)
+  ON_BN_CLICKED(IDC_USEUPPERCASE, OnUseUpperCase)
+  ON_BN_CLICKED(IDC_USEDIGITS, OnUseDigits)
+  ON_BN_CLICKED(IDC_USESYMBOLS, OnUseSymbols)
+  ON_BN_CLICKED(IDC_EASYVISION, OnEasyVision)
+  ON_BN_CLICKED(IDC_PRONOUNCEABLE, OnMakePronounceable)
 
-// Because we can show the generated password when used from Manage->Generate
-ON_BN_CLICKED(IDC_GENERATEPASSWORD, OnGeneratePassword)
-ON_BN_CLICKED(IDC_COPYPASSWORD, OnCopyPassword)
-ON_EN_CHANGE(IDC_PASSWORD, OnENChangePassword)
+  // Because we can show the generated password when used from Manage->Generate
+  ON_BN_CLICKED(IDC_GENERATEPASSWORD, OnGeneratePassword)
+  ON_BN_CLICKED(IDC_COPYPASSWORD, OnCopyPassword)
+  ON_EN_CHANGE(IDC_PASSWORD, OnENChangePassword)
 
-ON_BN_CLICKED(IDC_USENAMED_POLICY, OnUseNamedPolicy)
+  ON_BN_CLICKED(IDC_USENAMED_POLICY, OnUseNamedPolicy)
 
-ON_EN_KILLFOCUS(IDC_POLICYNAME, OnENChangePolicyName)
-ON_EN_KILLFOCUS(IDC_OWNSYMBOLS, OnENOwnSymbols)
+  ON_EN_KILLFOCUS(IDC_POLICYNAME, OnENChangePolicyName)
+  ON_EN_KILLFOCUS(IDC_OWNSYMBOLS, OnENOwnSymbols)
 
-ON_CBN_SELCHANGE(IDC_POLICYLIST, OnNamesComboChanged)
-//}}AFX_MSG_MAP
-ON_BN_CLICKED(IDC_RESET_SYMBOLS, &CPasswordPolicyDlg::OnSymbolReset)
+  ON_CBN_SELCHANGE(IDC_POLICYLIST, OnNamesComboChanged)
+  ON_BN_CLICKED(IDC_RESET_SYMBOLS, &CPasswordPolicyDlg::OnSymbolReset)
+  //}}AFX_MSG_MAP
 END_MESSAGE_MAP()
 
 /////////////////////////////////////////////////////////////////////////////
@@ -175,9 +176,7 @@ END_MESSAGE_MAP()
 // Following copied from AddEdit_PasswordPolicy.cpp. Move to common mixin?
 static void setupBuddy(CWnd *p, int spinid, int id, int &length, short min = 0)
 {
-  CSpinButtonCtrl* pspin;
-
-  pspin = (CSpinButtonCtrl *)p->GetDlgItem(spinid);
+  CSpinButtonCtrl* pspin = (CSpinButtonCtrl *)p->GetDlgItem(spinid);
   pspin->SetBuddy(p->GetDlgItem(id));
   pspin->SetRange(min, 1024);
   pspin->SetBase(10);
@@ -331,7 +330,7 @@ BOOL CPasswordPolicyDlg::OnInitDialog()
     }
 
     m_PolicyNameEdit.SetWindowText(m_policyname);
-    // Max. length of policy name is 255 - only 2 hex digits used for length
+    // Max. length of policy name is 255 - only one byte used for length
     // in database header
     m_PolicyNameEdit.SetLimitText(255);
     break;
@@ -374,6 +373,29 @@ BOOL CPasswordPolicyDlg::OnInitDialog()
   if (m_uicaller == IDS_GENERATEPASSWORD) {
     // Disable Specific policy controls as default is to use a named policy (database default)
     SetSpecificPolicyControls(FALSE);
+
+    m_pToolTipCtrl = new CToolTipCtrl;
+    if (!m_pToolTipCtrl->Create(this, TTS_ALWAYSTIP | TTS_BALLOON | TTS_NOPREFIX)) {
+      pws_os::Trace(L"Unable To create Advanced Dialog ToolTip\n");
+      delete m_pToolTipCtrl;
+      m_pToolTipCtrl = NULL;
+      return TRUE;
+    }
+
+    // Tooltips
+    EnableToolTips();
+
+    // Activate the tooltip control.
+    m_pToolTipCtrl->Activate(TRUE);
+    m_pToolTipCtrl->SetMaxTipWidth(300);
+    // Quadruple the time to allow reading by user
+    int iTime = m_pToolTipCtrl->GetDelayTime(TTDT_AUTOPOP);
+    m_pToolTipCtrl->SetDelayTime(TTDT_AUTOPOP, 4 * iTime);
+
+    // Set the tooltip text
+    CString cs_ToolTip;
+    cs_ToolTip.LoadString(IDS_CLICKTOCOPY);
+    m_pToolTipCtrl->AddTool(GetDlgItem(IDC_COPYPASSWORD), cs_ToolTip);
   }
 
   // Set appropriate focus
@@ -383,6 +405,9 @@ BOOL CPasswordPolicyDlg::OnInitDialog()
 
 BOOL CPasswordPolicyDlg::PreTranslateMessage(MSG* pMsg)
 {
+  if (m_pToolTipCtrl != NULL)
+    m_pToolTipCtrl->RelayEvent(pMsg);
+
   if (pMsg->message == WM_KEYDOWN && 
       (pMsg->wParam == VK_F1 || pMsg->wParam == VK_ESCAPE)) {
     PostMessage(WM_COMMAND, MAKELONG(pMsg->wParam == VK_F1 ? ID_HELP : IDCANCEL, BN_CLICKED), NULL);
@@ -433,7 +458,7 @@ void CPasswordPolicyDlg::OnCancel()
       m_PWEasyVision        != m_oldPWEasyVision        ||
       m_PWUseHexdigits      != m_oldPWUseHexdigits      ||
       m_PWMakePronounceable != m_oldPWMakePronounceable ||
-      m_Symbols            != m_oldSymbols) {
+      m_Symbols             != m_oldSymbols) {
     // Confirm Cancel from User
     CGeneralMsgBox gmb;
     if (gmb.AfxMessageBox(IDS_AREYOUSURE_OPT,
@@ -463,7 +488,7 @@ void CPasswordPolicyDlg::SetPolicyData(CString &cs_policyname,
   m_iter = m_policyname.IsEmpty() ? m_MapPSWDPLC.end() :
                                     m_MapPSWDPLC.find(StringX((LPCWSTR)m_policyname));
 
-  // Check the find worked above - if PolicyName not empty, it must be in the map!
+  // Check the find() worked above - if PolicyName not empty, it must be in the map!
   if (!m_policyname.IsEmpty())
     ASSERT(m_iter != m_MapPSWDPLC.end());
 
@@ -498,7 +523,8 @@ void CPasswordPolicyDlg::SetPolicyData(CString &cs_policyname,
   m_PWUpperMinLength = m_oldPWUpperMinLength = xst_pp.upperminlength;
 
   CString cs_symbols = xst_pp.symbols.c_str();
-  m_Symbols = m_oldSymbols = cs_symbols;
+  if (m_PWUseSymbols &&!cs_symbols.IsEmpty())
+    m_Symbols = m_oldSymbols = cs_symbols;
 }
 
 void CPasswordPolicyDlg::OnNamesComboChanged()
@@ -616,8 +642,8 @@ void CPasswordPolicyDlg::do_hex(const bool bHex)
 
 void CPasswordPolicyDlg::do_easyorpronounceable(const int iSet)
 {
-  // Can't have minimum lengths!
-  if ((m_PWEasyVision == TRUE  || m_PWMakePronounceable == TRUE) &&
+  // Can't have at-least for Pronounceables!
+  if ((m_PWMakePronounceable == TRUE) &&
       (m_PWDigitMinLength > 1  || m_PWLowerMinLength > 1 ||
        m_PWSymbolMinLength > 1 || m_PWUpperMinLength > 1)) {
     CGeneralMsgBox gmb;
@@ -626,37 +652,32 @@ void CPasswordPolicyDlg::do_easyorpronounceable(const int iSet)
 
   CString cs_value;
   int i;
-  if (iSet != EVPR_NONE) {
-    // Make EasyVision or Pronounceable
+  if (iSet == EVPR_PR) {
+    // Don't support at-least for Pronounceables
     for (i = 0; i < N_HEX_LENGTHS; i++) {
-      GetDlgItem(nonHexLengths[i])->ShowWindow(SW_HIDE);
-      GetDlgItem(nonHexLengthSpins[i])->ShowWindow(SW_HIDE);
-      GetDlgItem(LenTxts[2 * i])->ShowWindow(SW_HIDE);
-      GetDlgItem(LenTxts[2 * i + 1])->ShowWindow(SW_HIDE);
+      GetDlgItem(nonHexLengths[i])->EnableWindow(FALSE);
+      GetDlgItem(nonHexLengthSpins[i])->EnableWindow(FALSE);
+      GetDlgItem(LenTxts[2 * i])->EnableWindow(FALSE);
+      GetDlgItem(LenTxts[2 * i + 1])->EnableWindow(FALSE);
     }
 
     m_savelen[SAVE_LOWERCASE] = m_PWLowerMinLength;
     m_savelen[SAVE_UPPERCASE] = m_PWUpperMinLength;
     m_savelen[SAVE_DIGITS] = m_PWDigitMinLength;
     m_savelen[SAVE_SYMBOLS] = m_PWSymbolMinLength;
-  } else {
-    // Unmake EasyVision or Pronounceable
+  } else { // we're good for at-least
     for (i = 0; i < N_HEX_LENGTHS; i++) {
-      GetDlgItem(nonHexLengths[i])->ShowWindow(SW_SHOW);
-      GetDlgItem(nonHexLengthSpins[i])->ShowWindow(SW_SHOW);
-      GetDlgItem(LenTxts[2 * i])->ShowWindow(SW_SHOW);
-      GetDlgItem(LenTxts[2 * i + 1])->ShowWindow(SW_SHOW);
+      GetDlgItem(nonHexLengths[i])->EnableWindow(TRUE);
+      GetDlgItem(nonHexLengthSpins[i])->EnableWindow(TRUE);
+      GetDlgItem(LenTxts[2 * i])->EnableWindow(TRUE);
+      GetDlgItem(LenTxts[2 * i + 1])->EnableWindow(TRUE);
     }
-
+  }
+  if (iSet != EVPR_NONE) {
     BOOL bEnable = (IsDlgButtonChecked(IDC_USESYMBOLS) == BST_CHECKED) ? TRUE : FALSE;
     GetDlgItem(IDC_USESYMBOLS)->EnableWindow(TRUE);
     GetDlgItem(IDC_OWNSYMBOLS)->EnableWindow(bEnable);
     GetDlgItem(IDC_RESET_SYMBOLS)->EnableWindow(bEnable);
-
-    m_PWLowerMinLength = m_savelen[SAVE_LOWERCASE];
-    m_PWUpperMinLength = m_savelen[SAVE_UPPERCASE];
-    m_PWDigitMinLength = m_savelen[SAVE_DIGITS];
-    m_PWSymbolMinLength = m_savelen[SAVE_SYMBOLS];
   }
 }
 
@@ -798,17 +819,15 @@ BOOL CPasswordPolicyDlg::Validate()
   }
 
   if (!(m_PWUseHexdigits || m_PWEasyVision || m_PWMakePronounceable) &&
-      (m_PWDigitMinLength + m_PWLowerMinLength +
-       m_PWSymbolMinLength + m_PWUpperMinLength) > m_PWDefaultLength) {
+      ((m_PWUseDigits ? m_PWDigitMinLength : 0) +
+       (m_PWUseLowercase ? m_PWLowerMinLength : 0) +
+       (m_PWUseSymbols ? m_PWSymbolMinLength : 0) +
+       (m_PWUseUppercase ? m_PWUpperMinLength : 0)) > m_PWDefaultLength) {
     gmb.AfxMessageBox(IDS_DEFAULTPWLENGTHTOOSMALL);
     ((CEdit*)GetDlgItem(IDC_DEFPWLENGTH))->SetFocus();
     return FALSE;
   }
 
-  if ((m_PWUseHexdigits || m_PWEasyVision || m_PWMakePronounceable))
-    m_PWDigitMinLength = m_PWLowerMinLength =
-       m_PWSymbolMinLength = m_PWUpperMinLength = 1;
-
   if (m_uicaller == IDS_PSWDPOLICY) {
     // Only need to check if policy name is empty or it has been changed
     // from the old name and the new name is already defined
@@ -995,6 +1014,7 @@ void CPasswordPolicyDlg::SetSpecificPolicyControls(const BOOL bEnable)
 
     // Set up the correct controls (enabled/disabled)
     do_hex(m_PWUseHexdigits == TRUE);
+    GetDlgItem(IDC_USEHEXDIGITS)->EnableWindow(TRUE);
 
     int iSet = EVPR_NONE;
     if (m_PWEasyVision == TRUE)
