@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -20,7 +20,7 @@ IMPLEMENT_DYNAMIC(CWZPropertyPage, CPropertyPage)
 
 CWZPropertyPage::CWZPropertyPage(UINT nID, UINT nIDCaption, const int nType)
 : CPropertyPage(nID, nIDCaption), m_nID(nID), m_nType(nType),
-  m_pToolTipCtrl(NULL)
+m_pToolTipCtrl(NULL)
 {
 }
 
@@ -30,6 +30,7 @@ void CWZPropertyPage::DoDataExchange(CDataExchange* pDX)
 }
 
 BEGIN_MESSAGE_MAP(CWZPropertyPage, CPropertyPage)
+  ON_WM_CTLCOLOR()
   //{{AFX_MSG_MAP(CWZPropertyPage)
   //}}AFX_MSG_MAP
 END_MESSAGE_MAP()
@@ -95,18 +96,37 @@ void CWZPropertyPage::ShowHelp(const CString &topicFile)
 
 LRESULT CWZPropertyPage::WindowProc(UINT message, WPARAM wParam, LPARAM lParam)
 {
-  CWnd *pParent = GetParent();
-  while (pParent != NULL) {
-    DboxMain *pDbx = dynamic_cast<DboxMain *>(pParent);
-    if (pDbx != NULL && pDbx->m_eye_catcher != NULL &&
-        wcscmp(pDbx->m_eye_catcher, EYE_CATCHER) == 0) {
-      pDbx->ResetIdleLockCounter(message);
-      break;
+  if (app.GetMainDlg()->m_eye_catcher != NULL &&
+      wcscmp(app.GetMainDlg()->m_eye_catcher, EYE_CATCHER) == 0) {
+    app.GetMainDlg()->ResetIdleLockCounter(message);
     } else
-      pParent = pParent->GetParent();
-  }
-  if (pParent == NULL)
     pws_os::Trace(L"CWZPropertyPage::WindowProc - couldn't find DboxMain ancestor\n");
 
   return CPropertyPage::WindowProc(message, wParam, lParam);
 }
+
+BOOL CWZPropertyPage::PreTranslateMessage(MSG *pMsg)
+{
+  // Show/hide caps lock indicator
+  CWnd *pCapsLock = GetDlgItem(IDC_CAPSLOCK);
+  if (pCapsLock != NULL) {
+    pCapsLock->ShowWindow(((GetKeyState(VK_CAPITAL) & 0x0001) == 0x0001) ?
+                            SW_SHOW : SW_HIDE);
+  }
+
+  return CPropertyPage::PreTranslateMessage(pMsg);
+}
+
+HBRUSH CWZPropertyPage::OnCtlColor(CDC *pDC, CWnd *pWnd, UINT nCtlColor)
+{
+  HBRUSH hbr = CPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);
+
+  if (nCtlColor == CTLCOLOR_STATIC) {
+    UINT nID = pWnd->GetDlgCtrlID();
+    if (nID == IDC_CAPSLOCK) {
+      pDC->SetTextColor(RGB(255, 0, 0));
+      pDC->SetBkMode(TRANSPARENT);
+    }
+  }
+  return hbr;
+}
