@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -34,37 +34,37 @@
 #include <iomanip>
 #include <shlobj.h>
 
-
 using namespace std;
 
 const wchar_t CWZSelectDB::PSSWDCHAR = L'*';
 
 IMPLEMENT_DYNAMIC(CWZSelectDB, CWZPropertyPage)
 
-CWZSelectDB::CWZSelectDB(CWnd *pParent, UINT nIDCaption, const int nType)
- : CWZPropertyPage(IDD, nIDCaption, nType), m_tries(0), m_state(0),
+CWZSelectDB::CWZSelectDB(CWnd *pParent, int idd, UINT nIDCaption,
+       const int nType)
+ : CWZPropertyPage(idd, nIDCaption, nType), m_tries(0), m_state(0),
   m_pVKeyBoardDlg(NULL), m_bAdvanced(BST_UNCHECKED),
   m_bFileExistsUserAsked(false),
-  m_filespec(L""), m_passkey(L""),
+  m_filespec(L""), m_passkey(L""), m_passkey2(L""), m_verify2(L""),
   m_defexpdelim(L"\xbb"), m_pctlDB(new CEditExtn),
-  m_pctlPasskey(new CSecEditExtn)
+  m_pctlPasskey(new CSecEditExtn), m_pctlPasskey2(new CSecEditExtn), m_pctlVerify2(new CSecEditExtn),
+  m_LastFocus(IDC_PASSKEY)
 {
   m_pWZPSH = (CWZPropertySheet *)pParent;
-  if (pws_os::getenv("PWS_PW_MODE", false) == L"NORMAL")
+  if (pws_os::getenv("PWS_PW_MODE", false) == L"NORMAL") {
     m_pctlPasskey->SetSecure(false);
+    m_pctlPasskey2->SetSecure(false);
+    m_pctlVerify2->SetSecure(false);
+  }
   m_present = !IsYubiInserted(); // lie to trigger correct actions in timer event
-
-  // Call it as it also performs important initilisation
-  m_bVKAvailable = CVKeyBoardDlg::IsOSKAvailable();
-
-  m_bUseSecureDesktop = PWSprefs::GetInstance()->GetPref(PWSprefs::UseSecureDesktop);
-  m_iUserTimeLimit = PWSprefs::GetInstance()->GetPref(PWSprefs::SecureDesktopTimeout);
 }
 
 CWZSelectDB::~CWZSelectDB()
 {
   delete m_pctlDB;
   delete m_pctlPasskey;
+  delete m_pctlPasskey2;
+  delete m_pctlVerify2;
 
   if (m_pVKeyBoardDlg != NULL && m_pVKeyBoardDlg->SaveKLID()) {
     // Save Last Used Keyboard
@@ -75,6 +75,7 @@ CWZSelectDB::~CWZSelectDB()
     StringX cs_KLID = os.str().c_str();
     PWSprefs::GetInstance()->SetPref(PWSprefs::LastUsedKeyboard, cs_KLID);
 
+    m_pVKeyBoardDlg->DestroyWindow();
     delete m_pVKeyBoardDlg;
   }
 }
@@ -88,11 +89,14 @@ void CWZSelectDB::DoDataExchange(CDataExchange* pDX)
 
   DDX_Control(pDX, IDC_DATABASE, *m_pctlDB);
   DDX_Check(pDX, IDC_ADVANCED, m_bAdvanced);
-  DDX_Control(pDX, IDC_SD_TOGGLE, m_ctlSDToggle);
 
   // Can't use DDX_Text for CSecEditExtn
   m_pctlPasskey->DoDDX(pDX, m_passkey);
   DDX_Control(pDX, IDC_PASSKEY, *m_pctlPasskey);
+  m_pctlPasskey2->DoDDX(pDX, m_passkey2);
+  DDX_Control(pDX, IDC_PASSKEY2, *m_pctlPasskey2);
+  m_pctlVerify2->DoDDX(pDX, m_verify2);
+  DDX_Control(pDX, IDC_VERIFY2, *m_pctlVerify2);
 
   DDX_Control(pDX, IDC_YUBI_PROGRESS, m_yubi_timeout);
   DDX_Control(pDX, IDC_YUBI_STATUS, m_yubi_status);
@@ -102,8 +106,12 @@ void CWZSelectDB::DoDataExchange(CDataExchange* pDX)
   if (nID == ID_MENUITEM_SYNCHRONIZE         ||
       nID == ID_MENUITEM_EXPORT2PLAINTEXT    ||
       nID == ID_MENUITEM_EXPORTENT2PLAINTEXT ||
+      nID == ID_MENUITEM_EXPORTGRP2PLAINTEXT ||
       nID == ID_MENUITEM_EXPORT2XML          ||
-      nID == ID_MENUITEM_EXPORTENT2XML) {
+      nID == ID_MENUITEM_EXPORTENT2XML       ||
+      nID == ID_MENUITEM_EXPORTGRP2XML       ||
+      nID == ID_MENUITEM_EXPORTENT2DB        ||
+      nID == ID_MENUITEM_EXPORTGRP2DB) {
     DDX_Control(pDX, IDC_STATIC_WZWARNING, m_stc_warning);
 
     if (nID != ID_MENUITEM_SYNCHRONIZE) {
@@ -137,16 +145,24 @@ BEGIN_MESSAGE_MAP(CWZSelectDB, CWZPropertyPage)
   //{{AFX_MSG_MAP(CWZSelectDB)
   ON_WM_CTLCOLOR()
   ON_WM_TIMER()
-  ON_EN_CHANGE(IDC_PASSKEY, OnPassKeyChange)
   ON_EN_CHANGE(IDC_DATABASE, OnDatabaseChange)
+  ON_EN_CHANGE(IDC_PASSKEY, OnPassKeyChange)
+  ON_EN_CHANGE(IDC_PASSKEY2, OnPassKey2Change)
+  ON_EN_CHANGE(IDC_VERIFY2, OnVerify2Change)
+
+  ON_EN_SETFOCUS(IDC_PASSKEY, OnPasskeySetfocus)
+  ON_EN_SETFOCUS(IDC_PASSKEY2, OnPasskey2Setfocus)
+  ON_EN_SETFOCUS(IDC_VERIFY2, OnVerify2keySetfocus)
+
   ON_BN_CLICKED(IDC_BTN_BROWSE, OnOpenFileBrowser)
   ON_STN_CLICKED(IDC_VKB, OnVirtualKeyboard)
+  ON_STN_CLICKED(IDC_VKB2, OnVirtualKeyboard)
   ON_MESSAGE(PWS_MSG_INSERTBUFFER, OnInsertBuffer)
+
   ON_BN_CLICKED(ID_HELP, OnHelp)
   ON_BN_CLICKED(IDC_ADVANCED, OnAdvanced)
-  ON_BN_CLICKED(IDC_ENTERCOMBINATION, OnEnterCombination)
+
   ON_BN_CLICKED(IDC_YUBIKEY_BTN, OnYubikeyBtn)
-  ON_BN_CLICKED(IDC_SD_TOGGLE, OnSwitchSecureDesktop)
   //}}AFX_MSG_MAP
 END_MESSAGE_MAP()
 
@@ -159,27 +175,45 @@ BOOL CWZSelectDB::OnInitDialog()
 {
   CWZPropertyPage::OnInitDialog();
 
-  // Setup a timer to poll YubiKey every 250 ms
-  SetTimer(1, 250, 0);
-
-  // Configure dialog for Secure Dialog
-  ConfigureDialog();
+  SetTimer(1, 250, 0); // Setup a timer to poll YubiKey every 250 ms
 
   const UINT nID = m_pWZPSH->GetID();
+
+  Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_PASSKEY));
+  Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_PASSKEY2));
+  Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_VERIFY2));
+  m_pctlPasskey->SetPasswordChar(PSSWDCHAR);
+  m_pctlPasskey2->SetPasswordChar(PSSWDCHAR);
+  m_pctlVerify2->SetPasswordChar(PSSWDCHAR);
+
   CString cs_text,cs_temp;
 
-  bool bWARNINGTEXT(true);
+  bool bWARNINGTEXT(true), bEXPORTDBCTRLS(false);
   switch (nID) {
     case ID_MENUITEM_SYNCHRONIZE:
       cs_text.LoadString(IDS_WZSLCT_WARNING_SYNC);
       break;
     case ID_MENUITEM_EXPORT2PLAINTEXT:
-    case ID_MENUITEM_EXPORTENT2PLAINTEXT:
     case ID_MENUITEM_EXPORT2XML:
+      cs_temp.LoadString(IDS_WSSLCT_ALL);
+      break;
+    case ID_MENUITEM_EXPORTENT2PLAINTEXT:
     case ID_MENUITEM_EXPORTENT2XML:
-      cs_temp.LoadString((nID == ID_MENUITEM_EXPORT2PLAINTEXT || nID == ID_MENUITEM_EXPORT2XML) ?
-                              IDS_WSSLCT_ALL : IDS_WSSLCT_ENTRY);
-      cs_text.Format(IDS_WZSLCT_WARNING_EXP, cs_temp);
+    case ID_MENUITEM_EXPORTENT2DB:
+      cs_temp.LoadString(IDS_WSSLCT_ENTRY);
+      if (nID == ID_MENUITEM_EXPORTENT2DB) {
+        bWARNINGTEXT = false;
+        bEXPORTDBCTRLS = true;
+      }
+      break;
+    case ID_MENUITEM_EXPORTGRP2PLAINTEXT:
+    case ID_MENUITEM_EXPORTGRP2XML:
+    case ID_MENUITEM_EXPORTGRP2DB:
+      cs_temp.LoadString(IDS_WSSLCT_GROUP);
+      if (nID == ID_MENUITEM_EXPORTGRP2DB) {
+        bWARNINGTEXT = false;
+        bEXPORTDBCTRLS = true;
+      }
       break;
     case ID_MENUITEM_COMPARE:
     case ID_MENUITEM_MERGE:
@@ -190,10 +224,27 @@ BOOL CWZSelectDB::OnInitDialog()
       ASSERT(0);
   }
 
+  if (bEXPORTDBCTRLS) {
+    // Show & Enable Export Combination controls
+    GetDlgItem(IDC_STATIC_NEWCOMBI)->ShowWindow(SW_SHOW);
+    GetDlgItem(IDC_STATIC_VERIFY)->ShowWindow(SW_SHOW);
+    GetDlgItem(IDC_STATIC_COMBI)->ShowWindow(SW_SHOW);
+    GetDlgItem(IDC_PASSKEY2)->ShowWindow(SW_SHOW);
+    GetDlgItem(IDC_PASSKEY2)->EnableWindow(TRUE);
+    GetDlgItem(IDC_VERIFY2)->ShowWindow(SW_SHOW);
+    GetDlgItem(IDC_VERIFY2)->EnableWindow(TRUE);
+    GetDlgItem(IDC_VKB2)->ShowWindow(SW_SHOW);
+    GetDlgItem(IDC_VKB2)->EnableWindow(TRUE);
+  }
+
   if (bWARNINGTEXT) {
+    if (nID != ID_MENUITEM_SYNCHRONIZE)
+      cs_text.Format(IDS_WZSLCT_WARNING_EXP, cs_temp);
+
     GetDlgItem(IDC_STATIC_WZWARNING)->SetWindowText(cs_text);
     m_stc_warning.SetColour(RGB(255,0,0));
-    if (nID != ID_MENUITEM_EXPORT2PLAINTEXT && nID != ID_MENUITEM_EXPORTENT2PLAINTEXT)
+    if (nID != ID_MENUITEM_EXPORT2PLAINTEXT && nID != ID_MENUITEM_EXPORTENT2PLAINTEXT &&
+        nID != ID_MENUITEM_EXPORTGRP2PLAINTEXT)
       GetDlgItem(IDC_STATIC_WZEXPDLM2)->ShowWindow(SW_HIDE);
 
     LOGFONT LogFont;
@@ -208,20 +259,51 @@ BOOL CWZSelectDB::OnInitDialog()
 
   std::wstring ExportFileName;
   UINT uifilemsg(IDS_WZDATABASE);
+  stringT str_extn(L"");
   switch (nID) {
+    case ID_MENUITEM_EXPORTENT2DB:
+    case ID_MENUITEM_EXPORTGRP2DB:
+    {
+      // Disable & hide - Advnaced checkbox & specifying delimiter
+      GetDlgItem(IDC_ADVANCED)->ShowWindow(SW_HIDE);
+      GetDlgItem(IDC_ADVANCED)->EnableWindow(FALSE);
+
+      GetDlgItem(IDC_STATIC_WZEXPDLM1)->ShowWindow(SW_HIDE);
+      GetDlgItem(IDC_STATIC_WZEXPDLM2)->ShowWindow(SW_HIDE);
+      GetDlgItem(IDC_WZDEFEXPDELIM)->ShowWindow(SW_HIDE);
+      GetDlgItem(IDC_WZDEFEXPDELIM)->EnableWindow(FALSE);
+
+      stringT drive, dir, file, ext;
+      pws_os::splitpath(m_pWZPSH->WZPSHGetCurFile().c_str(), drive, dir, file, ext);
+
+      stringT str_file = file + L".export";
+
+      // Create new DB
+      ExportFileName = pws_os::makepath(drive, dir, str_file, L"psafe3");
+
+      m_pctlDB->SetWindowText(ExportFileName.c_str());
+      m_filespec = ExportFileName.c_str();
+      uifilemsg = IDS_WZFILE;
+      break;
+    }
     case ID_MENUITEM_EXPORT2XML:
     case ID_MENUITEM_EXPORTENT2XML:
-        GetDlgItem(IDC_STATIC_WZEXPDLM2)->ShowWindow(SW_HIDE);
-        // Drop though intentionally
+    case ID_MENUITEM_EXPORTGRP2XML:
+      str_extn = L"xml";
+
+      // Drop though intentionally
     case ID_MENUITEM_EXPORT2PLAINTEXT:
     case ID_MENUITEM_EXPORTENT2PLAINTEXT:
-        ExportFileName = PWSUtil::GetNewFileName(m_pWZPSH->WZPSHGetCurFile().c_str(),
-            (nID == ID_MENUITEM_EXPORT2XML || nID == ID_MENUITEM_EXPORTENT2XML) ?
-               L"xml" : L"txt");
-        m_pctlDB->SetWindowText(ExportFileName.c_str());
-        m_filespec = ExportFileName.c_str();
-        uifilemsg = IDS_WZFILE;
+    case ID_MENUITEM_EXPORTGRP2PLAINTEXT:
+      if (str_extn.empty())
+        str_extn = L"txt";
+      ExportFileName = PWSUtil::GetNewFileName(m_pWZPSH->WZPSHGetCurFile().c_str(),
+                                               str_extn.c_str());
+      m_pctlDB->SetWindowText(ExportFileName.c_str());
+      m_filespec = ExportFileName.c_str();
+      uifilemsg = IDS_WZFILE;
       break;
+
     case ID_MENUITEM_SYNCHRONIZE:
     case ID_MENUITEM_COMPARE:
     case ID_MENUITEM_MERGE:
@@ -240,9 +322,11 @@ BOOL CWZSelectDB::OnInitDialog()
   if (!CVKeyBoardDlg::IsOSKAvailable()) {
     GetDlgItem(IDC_VKB)->ShowWindow(SW_HIDE);
     GetDlgItem(IDC_VKB)->EnableWindow(FALSE);
+    GetDlgItem(IDC_VKB2)->ShowWindow(SW_HIDE);
+    GetDlgItem(IDC_VKB2)->EnableWindow(FALSE);
   }
 
-   // Disable Next until fields set
+  // Disable Next until fields set
   m_pWZPSH->SetWizardButtons(0);
 
   CString cs_tmp(MAKEINTRESOURCE(m_pWZPSH->GetButtonID()));
@@ -254,35 +338,31 @@ BOOL CWZSelectDB::OnInitDialog()
   m_yubiLogo.LoadBitmap(IDB_YUBI_LOGO);
   m_yubiLogoDisabled.LoadBitmap(IDB_YUBI_LOGO_DIS);
 
-  if (!m_bUseSecureDesktop) {
-    // Disable passphrase until database name filled in
-    m_pctlPasskey->EnableWindow(TRUE);
+  // Disable passphrase until database name filled in
+  m_pctlPasskey->EnableWindow(TRUE);
 
-    CWnd *ybn = GetDlgItem(IDC_YUBIKEY_BTN);
+  CWnd *ybn = GetDlgItem(IDC_YUBIKEY_BTN);
 
-    if (YubiExists()) {
-      ybn->ShowWindow(SW_SHOW);
-      m_yubi_status.ShowWindow(SW_SHOW);
-    }
-    else {
-      ybn->ShowWindow(SW_HIDE);
-      m_yubi_status.ShowWindow(SW_HIDE);
-    }
-    m_yubi_timeout.ShowWindow(SW_HIDE);
-    m_yubi_timeout.SetRange(0, 15);
-    bool bYubiInserted = IsYubiInserted();
-    // MFC has ancient bug: can't render diasbled version of bitmap,
-    // so instead of showing drek, we roll our own, and leave enabled.
-    ybn->EnableWindow(TRUE);
+  if (YubiExists()) {
+    ybn->ShowWindow(SW_SHOW);
+    m_yubi_status.ShowWindow(SW_SHOW);
+  } else {
+    ybn->ShowWindow(SW_HIDE);
+    m_yubi_status.ShowWindow(SW_HIDE);
+  }
+  m_yubi_timeout.ShowWindow(SW_HIDE);
+  m_yubi_timeout.SetRange(0, 15);
+  bool bYubiInserted = IsYubiInserted();
+  // MFC has ancient bug: can't render diasbled version of bitmap,
+  // so instead of showing drek, we roll our own, and leave enabled.
+  ybn->EnableWindow(TRUE);
 
-    if (bYubiInserted) {
-      ((CButton*)ybn)->SetBitmap(m_yubiLogo);
-      m_yubi_status.SetWindowText(CString(MAKEINTRESOURCE(IDS_YUBI_CLICK_PROMPT)));
-    }
-    else {
-      ((CButton*)ybn)->SetBitmap(m_yubiLogoDisabled);
-      m_yubi_status.SetWindowText(CString(MAKEINTRESOURCE(IDS_YUBI_INSERT_PROMPT)));
-    }
+  if (bYubiInserted) {
+    ((CButton*)ybn)->SetBitmap(m_yubiLogo);
+    m_yubi_status.SetWindowText(CString(MAKEINTRESOURCE(IDS_YUBI_CLICK_PROMPT)));
+  } else {
+    ((CButton*)ybn)->SetBitmap(m_yubiLogoDisabled);
+    m_yubi_status.SetWindowText(CString(MAKEINTRESOURCE(IDS_YUBI_INSERT_PROMPT)));
   }
 
   return FALSE;
@@ -331,63 +411,6 @@ HBRUSH CWZSelectDB::OnCtlColor(CDC *pDC, CWnd *pWnd, UINT nCtlColor)
   return hbr;
 }
 
-void CWZSelectDB::OnSwitchSecureDesktop()
-{
-  m_bUseSecureDesktop = !m_bUseSecureDesktop;
-
-  ConfigureDialog();
-}
-
-void CWZSelectDB::ConfigureDialog()
-{
-  // This bit makes the background come out right on the bitmaps - these 2 bitmaps use white as the mask
-  m_ctlSDToggle.SetBitmapMaskAndID(RGB(255, 255, 255), m_bUseSecureDesktop ? IDB_USING_SD : IDB_NOT_USING_SD);
-  m_ctlSDToggle.Invalidate();
-
-  if (!m_bUseSecureDesktop)
-  {
-    Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_PASSKEY));
-    m_pctlPasskey->SetPasswordChar(PSSWDCHAR);
-
-    GetDlgItem(IDC_ENTERCOMBINATION)->ShowWindow(SW_HIDE);
-    GetDlgItem(IDC_ENTERCOMBINATION)->EnableWindow(FALSE);
-    GetDlgItem(IDC_STATIC_ENTERCOMBINATION)->ShowWindow(SW_SHOW);
-
-    GetDlgItem(IDC_PASSKEY)->ShowWindow(SW_SHOW);
-    GetDlgItem(IDC_PASSKEY)->EnableWindow(TRUE);
-    GetDlgItem(IDC_VKB)->ShowWindow(SW_SHOW);
-    GetDlgItem(IDC_VKB)->EnableWindow(TRUE);
-
-    if (IsYubiInserted())
-    {
-      GetDlgItem(IDC_YUBIKEY_BTN)->ShowWindow(SW_SHOW);
-      GetDlgItem(IDC_YUBIKEY_BTN)->EnableWindow(TRUE);
-      GetDlgItem(IDC_YUBI_PROGRESS)->ShowWindow(SW_SHOW);
-      GetDlgItem(IDC_YUBI_PROGRESS)->EnableWindow(TRUE);
-      GetDlgItem(IDC_YUBI_STATUS)->ShowWindow(SW_SHOW);
-      GetDlgItem(IDC_YUBI_STATUS)->EnableWindow(TRUE);
-    }
-  }
-  else
-  {
-    GetDlgItem(IDC_ENTERCOMBINATION)->ShowWindow(SW_SHOW);
-    GetDlgItem(IDC_ENTERCOMBINATION)->EnableWindow(TRUE);
-    GetDlgItem(IDC_STATIC_ENTERCOMBINATION)->ShowWindow(SW_HIDE);
-
-    GetDlgItem(IDC_PASSKEY)->ShowWindow(SW_HIDE);
-    GetDlgItem(IDC_PASSKEY)->EnableWindow(FALSE);
-    GetDlgItem(IDC_VKB)->ShowWindow(SW_HIDE);
-    GetDlgItem(IDC_VKB)->EnableWindow(FALSE);
-
-    GetDlgItem(IDC_YUBIKEY_BTN)->ShowWindow(SW_HIDE);
-    GetDlgItem(IDC_YUBIKEY_BTN)->EnableWindow(FALSE);
-    GetDlgItem(IDC_YUBI_PROGRESS)->ShowWindow(SW_HIDE);
-    GetDlgItem(IDC_YUBI_PROGRESS)->EnableWindow(FALSE);
-    GetDlgItem(IDC_YUBI_STATUS)->ShowWindow(SW_HIDE);
-    GetDlgItem(IDC_YUBI_STATUS)->EnableWindow(FALSE);
-  }
-}
-
 BOOL CWZSelectDB::OnSetActive()
 {
   CWZPropertyPage::OnSetActive();
@@ -397,6 +420,22 @@ BOOL CWZSelectDB::OnSetActive()
   return TRUE;
 }
 
+void CWZSelectDB::OnPasskeySetfocus()
+{
+  m_LastFocus = IDC_PASSKEY;
+}
+
+void CWZSelectDB::OnPasskey2Setfocus()
+{
+  m_LastFocus = IDC_PASSKEY2;
+}
+
+void CWZSelectDB::OnVerify2keySetfocus()
+{
+  m_LastFocus = IDC_VERIFY2;
+}
+
+
 void CWZSelectDB::OnAdvanced()
 {
   //m_bAdvanced = ((CButton*)GetDlgItem(IDC_ADVANCED))->GetCheck();
@@ -408,47 +447,70 @@ void CWZSelectDB::OnAdvanced()
   m_pWZPSH->GetDlgItem(ID_WIZNEXT)->SetWindowText(cs_tmp);
 }
 
-void CWZSelectDB::OnEnterCombination()
-{
-  // Only needed for its thread processing - never dispays its own dialog (no DoModal etc.)
-  CPKBaseDlg PKBaseDlg(IDD_PASSKEYENTRY_SD, this, true);
-
-  // Avoid polling Yubikey from > 1 thread
-  m_yubiPollDisable = true;
-
-  // Get current monitor - not possible in PKBaseDlg as its dialog is not shown
-  HMONITOR hCurrentMonitor = MonitorFromWindow(this->GetSafeHwnd(), MONITOR_DEFAULTTONEAREST);
-
-  // Get passphrase from Secure Desktop
-  PKBaseDlg.StartThread(IDD_SDGETPHRASE, hCurrentMonitor);
-
-  if (PKBaseDlg.m_GMP.bPhraseEntered) {
-    m_passkey = PKBaseDlg.m_GMP.sPhrase.c_str();
-    if (m_passkey.GetLength() > 0)
-      m_state |= KEYPRESENT;
-    else
-      m_state &= ~KEYPRESENT;
-
-    m_pWZPSH->SetWizardButtons(m_state == BOTHPRESENT ? PSWIZB_NEXT : 0);
-  }
-
-  ShowWindow(SW_SHOW);
-  UpdateData(FALSE);
-}
-
 void CWZSelectDB::OnPassKeyChange()
 {
-  CString cs_Passkey;
-  m_pctlPasskey->GetWindowText(cs_Passkey);
+  CSecString cs_Passkey;
+  cs_Passkey =  m_pctlPasskey->GetSecureText();
 
-  if (cs_Passkey.GetLength() > 0)
+  if (!cs_Passkey.IsEmpty())
     m_state |= KEYPRESENT;
   else
     m_state &= ~KEYPRESENT;
 
-  m_passkey = CSecString(cs_Passkey);
+  m_passkey = cs_Passkey;
 
-  m_pWZPSH->SetWizardButtons(m_state == BOTHPRESENT ? PSWIZB_NEXT : 0);
+  const UINT nID = m_pWZPSH->GetID();
+  bool bReady;
+
+  if (nID == ID_MENUITEM_EXPORTENT2DB || nID == ID_MENUITEM_EXPORTGRP2DB) {
+    bReady = m_state == ALLPRESENT;
+  } else {
+    bReady = m_state == BOTHPRESENT;
+  }
+
+  m_pWZPSH->SetWizardButtons(bReady ? PSWIZB_NEXT : 0);
+}
+
+void CWZSelectDB::OnPassKey2Change()
+{
+  CSecString cs_Passkey2, cs_Verify2;
+  cs_Passkey2 = m_pctlPasskey2->GetSecureText();
+  cs_Verify2 = m_pctlVerify2->GetSecureText();
+
+  if (!cs_Passkey2.IsEmpty())
+    m_state |= KEY2PRESENT;
+  else
+    m_state &= ~KEY2PRESENT;
+
+  if (!cs_Verify2.IsEmpty() && cs_Passkey2 == cs_Verify2)
+    m_state |= KEY2_EQ_VERIFY2;
+  else
+    m_state &= ~KEY2_EQ_VERIFY2;
+
+  m_passkey2 = cs_Passkey2;
+
+  m_pWZPSH->SetWizardButtons(m_state == ALLPRESENT ? PSWIZB_NEXT : 0);
+}
+
+void CWZSelectDB::OnVerify2Change()
+{
+  CSecString cs_Passkey2, cs_Verify2;
+  cs_Passkey2 = m_pctlPasskey2->GetSecureText();
+  cs_Verify2 = m_pctlVerify2->GetSecureText();
+
+  if (!cs_Verify2.IsEmpty())
+    m_state |= VERIFY2PRESENT;
+  else
+    m_state &= ~VERIFY2PRESENT;
+
+  if (!cs_Passkey2.IsEmpty() && cs_Passkey2 == cs_Verify2)
+    m_state |= KEY2_EQ_VERIFY2;
+  else
+    m_state &= ~KEY2_EQ_VERIFY2;
+
+  m_verify2 = cs_Verify2;
+
+  m_pWZPSH->SetWizardButtons(m_state == ALLPRESENT ? PSWIZB_NEXT : 0);
 }
 
 void CWZSelectDB::OnDatabaseChange()
@@ -456,7 +518,7 @@ void CWZSelectDB::OnDatabaseChange()
   CString cs_DB;
   m_pctlDB->GetWindowText(cs_DB);
 
-  if (cs_DB.GetLength() > 0)
+  if (!cs_DB.IsEmpty())
     m_state |= DBPRESENT;
   else
     m_state &= ~DBPRESENT;
@@ -474,7 +536,7 @@ LRESULT CWZSelectDB::OnWizardNext()
 
   const UINT nID = m_pWZPSH->GetID();
   bool bFileExists = pws_os::FileExists(m_filespec.GetString());
-  bool bExportXML(true);
+  int iExportType(-1);  // -1 = Not set, IDS_WZEXPORTTEXT = Text, IDS_WZEXPORTXML = XML, IDS_WZEXPORTDB = DB
 
   switch (nID) {
     case ID_MENUITEM_COMPARE:
@@ -489,14 +551,22 @@ LRESULT CWZSelectDB::OnWizardNext()
       break;
     case ID_MENUITEM_EXPORT2PLAINTEXT:
     case ID_MENUITEM_EXPORTENT2PLAINTEXT:
-      bExportXML = false; // Fall through on purpose
+    case ID_MENUITEM_EXPORTGRP2PLAINTEXT:
+      iExportType = IDS_WZEXPORTTEXT; // Fall through on purpose
     case ID_MENUITEM_EXPORT2XML:
     case ID_MENUITEM_EXPORTENT2XML:
+    case ID_MENUITEM_EXPORTGRP2XML:
+      if (iExportType < 0)
+        iExportType = IDS_WZEXPORTXML;
+    case ID_MENUITEM_EXPORTENT2DB:
+    case ID_MENUITEM_EXPORTGRP2DB:
+      if (iExportType < 0)
+        iExportType = IDS_WZEXPORTDB;
       if (bFileExists) {
         // Check if OK to overwrite existing file - if not already asked by user clicking
         // file browser button
         if (!m_bFileExistsUserAsked) {
-          CString cs_msg, cs_title(MAKEINTRESOURCE(bExportXML ? IDS_WZEXPORTXML : IDS_WZEXPORTTEXT));
+          CString cs_msg, cs_title(MAKEINTRESOURCE(iExportType));
           cs_msg.Format(IDS_REPLACEEXPORTFILE, m_filespec);
           INT_PTR rc = gmb.AfxMessageBox(cs_msg, cs_title,
                          MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2);
@@ -535,16 +605,35 @@ LRESULT CWZSelectDB::OnWizardNext()
       return -1;
   }
 
+  // Current DB passkey
   if (m_passkey.IsEmpty()) {
     gmb.AfxMessageBox(IDS_CANNOTBEBLANK);
     m_pctlPasskey->SetFocus();
     return -1;
   }
 
+  if (nID == ID_MENUITEM_EXPORTENT2DB || nID == ID_MENUITEM_EXPORTGRP2DB) {
+    // New Export DB passkey
+    if (m_passkey2 != m_verify2) {
+      // This shouldn't happen as the Wizard Next button shouldn't be
+      // active unless they are equal!
+      gmb.AfxMessageBox(IDS_ENTRIESDONOTMATCH);
+      ((CEdit*)GetDlgItem(IDC_VERIFY2))->SetFocus();
+      return -1;
+    }
+
+    if (m_passkey2.IsEmpty()) {
+      gmb.AfxMessageBox(IDS_ENTERKEYANDVERIFY);
+      ((CEdit*)GetDlgItem(IDC_PASSKEY2))->SetFocus();
+      return -1;
+    }
+  }
+
   // Check that this file isn't already open
   const StringX sx_Filename1(m_pWZPSH->WZPSHGetCurFile());
   const StringX sx_Filename2 = m_filespec.GetString();
   const StringX sx_passkey = (LPCWSTR)m_passkey;
+  const StringX sx_exportpasskey = (LPCWSTR)m_passkey2;
   const bool bOtherIsDB = nID == ID_MENUITEM_COMPARE ||
                           nID == ID_MENUITEM_MERGE   ||
                           nID == ID_MENUITEM_SYNCHRONIZE;
@@ -559,6 +648,7 @@ LRESULT CWZSelectDB::OnWizardNext()
     return -1;
 
   m_pWZPSH->SetPassKey(sx_passkey);
+  m_pWZPSH->SetExportPassKey(sx_exportpasskey);
   m_pWZPSH->SetOtherDB(sx_Filename2);
   m_pWZPSH->SetDelimiter(m_defexpdelim.GetAt(0));
   m_pWZPSH->SetAdvanced(m_bAdvanced == BST_CHECKED);
@@ -610,6 +700,7 @@ void CWZSelectDB::OnOpenFileBrowser()
       break;
     case ID_MENUITEM_EXPORT2PLAINTEXT:
     case ID_MENUITEM_EXPORTENT2PLAINTEXT:
+    case ID_MENUITEM_EXPORTGRP2PLAINTEXT:
       cs_suffix = L"txt";
       cs_filter.LoadString(IDS_FDF_T_C_ALL);
       dwflags = OFN_PATHMUSTEXIST | OFN_HIDEREADONLY |
@@ -618,12 +709,21 @@ void CWZSelectDB::OnOpenFileBrowser()
       break;
     case ID_MENUITEM_EXPORT2XML:
     case ID_MENUITEM_EXPORTENT2XML:
+    case ID_MENUITEM_EXPORTGRP2XML:
       cs_suffix = L"xml";
       cs_filter.LoadString(IDS_FDF_X_ALL);
       dwflags = OFN_PATHMUSTEXIST | OFN_HIDEREADONLY |
                             OFN_LONGNAMES | OFN_OVERWRITEPROMPT;
       uimsgid = IDS_NAMEXMLFILE;
       break;
+    case ID_MENUITEM_EXPORTENT2DB:
+    case ID_MENUITEM_EXPORTGRP2DB:
+      cs_suffix = L"export";
+      cs_filter.LoadString(IDS_FDF_V3_ALL);
+      dwflags = OFN_PATHMUSTEXIST | OFN_HIDEREADONLY |
+        OFN_LONGNAMES | OFN_OVERWRITEPROMPT;
+      uimsgid = IDS_NAMEXMLFILE;
+      break;
     default:
       ASSERT(0);
   }
@@ -667,12 +767,10 @@ void CWZSelectDB::OnOpenFileBrowser()
   if (rc == IDOK) {
     m_filespec = fd.GetPathName();
     m_pctlDB->SetWindowText(m_filespec);
-    if (!m_bUseSecureDesktop) {
       m_pctlPasskey->EnableWindow(TRUE);
       if (m_pctlPasskey->IsWindowEnabled() == TRUE) {
         m_pctlPasskey->SetFocus();
       }
-    }
     // If the file exists and we are doing a save, CFileDialog
     // would have prompted the user
     if (bTYPE_OPEN == FALSE && pws_os::FileExists(m_filespec.GetString()))
@@ -682,38 +780,29 @@ void CWZSelectDB::OnOpenFileBrowser()
 
 void CWZSelectDB::OnVirtualKeyboard()
 {
-  // This is used if Secure Desktop isn't!
-  DWORD dwError; //  Define it here to stop warning that local variable is initialized but not referenced later on
-
   // Shouldn't be here if couldn't load DLL. Static control disabled/hidden
   if (!CVKeyBoardDlg::IsOSKAvailable())
     return;
 
-  if (m_hwndVKeyBoard != NULL && ::IsWindowVisible(m_hwndVKeyBoard)) {
+  if (m_pVKeyBoardDlg != NULL && m_pVKeyBoardDlg->IsWindowVisible()) {
     // Already there - move to top
-    ::SetWindowPos(m_hwndVKeyBoard, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
+    m_pVKeyBoardDlg->SetWindowPos(&wndTop , 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
     return;
   }
 
   // If not already created - do it, otherwise just reset it
   if (m_pVKeyBoardDlg == NULL) {
     StringX cs_LUKBD = PWSprefs::GetInstance()->GetPref(PWSprefs::LastUsedKeyboard);
-    m_pVKeyBoardDlg = new CVKeyBoardDlg(this->GetSafeHwnd(), this->GetSafeHwnd(), cs_LUKBD.c_str());
-    m_hwndVKeyBoard = CreateDialogParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_SDVKEYBOARD), this->GetSafeHwnd(),
-      (DLGPROC)(m_pVKeyBoardDlg->VKDialogProc), (LPARAM)(m_pVKeyBoardDlg));
-
-    if (m_hwndVKeyBoard == NULL) {
-      dwError = pws_os::IssueError(_T("CreateDialogParam - IDD_SDVKEYBOARD"), false);
-      ASSERT(m_hwndVKeyBoard);
-    }
-  }
-  else {
+    m_pVKeyBoardDlg = new CVKeyBoardDlg(this, cs_LUKBD.c_str());
+    m_pVKeyBoardDlg->Create(CVKeyBoardDlg::IDD);
+  } else {
     m_pVKeyBoardDlg->ResetKeyboard();
   }
 
-  // Now show it and make it top & enable it
-  ::SetWindowPos(m_hwndVKeyBoard, HWND_TOP, 0, 0, 0, 0, SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE);
-  ::EnableWindow(m_hwndVKeyBoard, TRUE);
+  // Now show it and make it top
+  m_pVKeyBoardDlg->SetWindowPos(&wndTop , 0, 0, 0, 0, SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE);
+
+  return;
 }
 
 LRESULT CWZSelectDB::OnInsertBuffer(WPARAM, LPARAM)
@@ -724,27 +813,67 @@ LRESULT CWZSelectDB::OnInsertBuffer(WPARAM, LPARAM)
   // Get the buffer
   CSecString vkbuffer = m_pVKeyBoardDlg->GetPassphrase();
 
+  CSecEditExtn *m_pSecCtl(NULL);
+  CSecString *m_pSecString;
+
+  switch (m_LastFocus) {
+    case IDC_PASSKEY:
+      m_pSecCtl = m_pctlPasskey;
+      m_pSecString = &m_passkey;
+      break;
+    case IDC_PASSKEY2:
+      m_pSecCtl = m_pctlPasskey2;
+      m_pSecString = &m_passkey2;
+      break;
+    case IDC_VERIFY2:
+      m_pSecCtl = m_pctlVerify2;
+      m_pSecString = &m_verify2;
+      break;
+    default:
+      // Error!
+      ASSERT(0);
+      return 0L;
+  }
+
   // Find the selected characters - if any
   int nStartChar, nEndChar;
-  m_pctlPasskey->GetSel(nStartChar, nEndChar);
+  m_pSecCtl->GetSel(nStartChar, nEndChar);
 
   // If any characters selected - delete them
   if (nStartChar != nEndChar)
-    m_passkey.Delete(nStartChar, nEndChar - nStartChar);
+    m_pSecString->Delete(nStartChar, nEndChar - nStartChar);
 
   // Insert the buffer
-  m_passkey.Insert(nStartChar, vkbuffer);
+  m_pSecString->Insert(nStartChar, vkbuffer);
 
   // Put cursor at end of inserted text
-  m_pctlPasskey->SetSel(nStartChar + vkbuffer.GetLength(),
-    nStartChar + vkbuffer.GetLength());
+  m_pSecCtl->SetSel(nStartChar + vkbuffer.GetLength(),
+                    nStartChar + vkbuffer.GetLength());
 
   // Update the dialog
   UpdateData(FALSE);
 
   // Ensure flags set so that buttons are activated as required
-  if (vkbuffer.GetLength() > 0)
-    OnPassKeyChange();
+  if (!vkbuffer.IsEmpty()) {
+    switch (m_LastFocus) {
+      case IDC_PASSKEY:
+        OnPassKeyChange();
+        break;
+      case IDC_PASSKEY2:
+        OnPassKey2Change();
+        break;
+      case IDC_VERIFY2:
+        OnVerify2Change();;
+        break;
+      default:
+        // Error!
+        ASSERT(0);
+        return 0L;
+    }
+  }
+
+  // Make us on top
+  SetWindowPos(&wndTop, 0, 0, 0, 0, SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE);
 
   return 0L;
 }
@@ -764,7 +893,6 @@ void CWZSelectDB::yubiRemoved(void)
   ((CButton*)GetDlgItem(IDC_YUBIKEY_BTN))->SetBitmap(m_yubiLogoDisabled);
   m_yubi_status.SetWindowText(CString(MAKEINTRESOURCE(IDS_YUBI_INSERT_PROMPT)));
 }
-  BYTE respBuf[SHA1_DIGEST_SIZE];
 
 void CWZSelectDB::yubiShowChallengeSent()
 {
@@ -786,6 +914,7 @@ void CWZSelectDB::yubiProcessCompleted(YKLIB_RC yrc, unsigned short ts, const BY
     TRACE(_T("yubiCheckCompleted: YKLIB_OK"));
     m_pending = false;
     m_passkey = Bin2Hex(respBuf, SHA1_DIGEST_SIZE);
+    m_state |= KEYPRESENT;
     // The returned hash is the passkey
     m_pWZPSH->SetWizardButtons(PSWIZB_NEXT); // enable
     // This will check the password, etc.:
@@ -799,6 +928,7 @@ void CWZSelectDB::yubiProcessCompleted(YKLIB_RC yrc, unsigned short ts, const BY
     break;
 
   case YKLIB_INVALID_RESPONSE:  // Invalid or no response
+    m_state &= ~KEYPRESENT;
     m_pending = false;
     m_yubi_timeout.ShowWindow(SW_HIDE);
     m_yubi_status.SetWindowText(CString(MAKEINTRESOURCE(IDS_YUBI_TIMEOUT)));
@@ -806,6 +936,7 @@ void CWZSelectDB::yubiProcessCompleted(YKLIB_RC yrc, unsigned short ts, const BY
     break;
 
   default:                // A non-recoverable error has occured
+    m_state &= ~KEYPRESENT;
     m_pending = false;
     m_yubi_timeout.ShowWindow(SW_HIDE);
     m_yubi_status.ShowWindow(SW_SHOW);
@@ -824,10 +955,6 @@ void CWZSelectDB::OnYubikeyBtn()
 
 void CWZSelectDB::OnTimer(UINT_PTR)
 {
-  // Ignore if Secure Desktop
-  if (m_bUseSecureDesktop)
-    return;
-
   // If an operation is pending, check if it has completed
   if (m_pending) {
     yubiCheckCompleted();
