@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2013 Rony Shapiro <ronys@users.sourceforge.net>.
+ * Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -63,6 +63,9 @@ DbSelectionPanel::DbSelectionPanel(wxWindow* parent,
                                           validator);
   panelSizer->Add(m_filepicker, borderFlags.Expand());
   panelSizer->AddSpacer(RowSeparation*rowsep);
+  m_filepicker->Connect( m_filepicker->GetEventType(), 
+             wxFileDirPickerEventHandler(DbSelectionPanel::OnFilePicked),
+             NULL, this);
 
   panelSizer->Add(new wxStaticText(this, wxID_ANY, _("Safe Combination:")), borderFlags);
   panelSizer->AddSpacer(RowSeparation);
@@ -122,3 +125,10 @@ bool DbSelectionPanel::DoValidation()
     return false;
   }
 }
+
+void DbSelectionPanel::OnFilePicked(wxFileDirPickerEvent& /* event */)
+{
+  // Don't shift focus if we are in the text ctrl
+  if ( !wxDynamicCast(FindFocus(), wxTextCtrl) )
+    m_sc->SelectCombinationText();
+}
